/*
 Nokia 3120 KSS (Keypad SMS Sender)
 Version: 1.0
 Author: Alex from insideGadgets (http://www.insidegadgets.com)
 Created: 26/12/2011
 Last Modified: 29/12/2011
 
 Uses the Nokia 3120 keypad to send an SMS.
 
 */
 
// ATtiny24/44/84 Pin map
//
//               +-\/-+
//         VCC  1|o   |14  GND
//         PB0  2|    |13  PA0 (ADC0)
//         PB1  3|    |12  PA1 (ADC1)
//   Reset PB3  4|    |11  PA2 (ADC2)
//         PB2  5|    |10  PA3 (ADC3)
//  (ADC7) PA7  6|    |9   PA4 (ADC4)
//  (ADC6) PA6  7|    |8   PA5 (ADC5)
//               +----+
//
//
// Nokia 3120 Button map
//
//					On			PB0		PB1		PB2		PB2
//					1			3			4			7			6
// PA0	2		Select	Up			Exit		Down
// PA1	5					Dial					Hangup
// PA2	9		But2					But0		But8
// PA3	10		But3		But6		Hash		But9
// PA4	8		But1		But4		Star		But7
// PA5	11														Left
// PA6	12														Right
// PA7	13														But5

#define F_CPU 1000000 // 1 MHz clock

#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/sleep.h>
#include <util/delay.h>
#include <avr/pgmspace.h>
#include "setup.c"

prog_char smsMessage[] PROGMEM = "hi this is a test sms which is typed using the keypad. by insidegadgets.";
prog_char mobileNumber[] PROGMEM = "0412345678";

// Break up the button number, turn on the necessary outputs and NPN transistor, delay and then turn off
void pushButton(char buttonNumber) {
	int npnOn = (buttonNumber % 10) - 2; // If reads -1, don't turn any transistors on as we use the one already on by default
	int outputOn = (buttonNumber / 10) - 1;
	
	if (npnOn >= 0) { 
		PORTB |= (1<<npnOn);
	}
	PORTA = (1<<outputOn);
	_delay_ms(100);
	
	PORTA = 0;
	if (npnOn >= 0) { 
		PORTB &= ~(1<<npnOn);
	}
	_delay_ms(100);
}

// Find which button to press for the character received
void parseChar(char c, byte onePushnumbers) {
	
	// Convert lowercase a-z to uppercase
	if (c >= 97 && c <= 122) {
		c -= 32;
	}
	
	// A-Z charset and 0-9 charset
	if ((c >= 65 && c <= 90) || (c >= 48 && c <= 57)) {
		byte lookupChar;
		byte loopEnd;
		byte buttonPressing;
		
		if (c >= 65 && c <= 90) {// A-Z charset
			lookupChar = c - 65;
			loopEnd = letterTobutton[lookupChar][buttonPushes];
			buttonPressing = letterTobutton[lookupChar][buttonTopush];
		}
		else if (c >= 48 && c <= 57) { //0-9 charset
			lookupChar = c - 48;
			loopEnd = numberTobutton[lookupChar][buttonPushes];
			buttonPressing = numberTobutton[lookupChar][buttonTopush];
		}
		
		// If we are in a section that allows numbers to be pressed just once (like dialing a number)
		if (onePushnumbers == true) {
			pushButton(buttonPressing);
		}
		else {
			for (int x = 0; x < loopEnd; x++) { // Push the corresponding button x times
				pushButton(buttonPressing);
			}
		}
	}
	
	// Dot
	else if (c == 46) {
		pushButton(button1);
	}
	
	// Space
	else if (c == 32) {
		pushButton(button0);
	}
}

// Send SMS message
void sendSMS(char *smsMsg, char *phoneNumber) {
	
	// Exit out anything that might be up
	pushButton(buttonHangup);
	_delay_ms(500);
	
	// Create new SMS message
	pushButton(buttonSelect);
	pushButton(buttonSelect);
	pushButton(buttonSelect);
	pushButton(buttonSelect);
	_delay_ms(500);
	
	// Parse message
	int charAt = 0;
	char c;
	while ((c = (char)pgm_read_byte(&smsMsg[charAt]))) {
		parseChar(c, 0);
		charAt++;
		pushButton(buttonRight);
	}
	
	// Enter phone number
	pushButton(buttonSelect);
	pushButton(buttonSelect);
	_delay_ms(500);
	
	// Parse phone number
	charAt = 0;
	while ((c = (char)pgm_read_byte(&phoneNumber[charAt]))) {
		parseChar(c, numbersPushedonce);
		charAt++;
	}
	
	// Send
	pushButton(buttonSelect);
}

int main(void) {
	
	setup();
	
	// Set Outputs
	DDRA = 0xFF;
	DDRB |= ((1<<PB0) | (1<<PB1) | (1<<PB2));
	
	while(1) {
		sendSMS(smsMessage, mobileNumber);
		system_sleep();
	}
}
